from .styling import Styling

class Background:
    @staticmethod
    def black():
        return "\033[40m" if Styling.is_enabled() else ""
        
    @staticmethod
    def red():
        return "\033[41m" if Styling.is_enabled() else ""
        
    @staticmethod
    def green():
        return "\033[42m" if Styling.is_enabled() else ""
        
    @staticmethod
    def yellow():
        return "\033[43m" if Styling.is_enabled() else ""
        
    @staticmethod
    def blue():
        return "\033[44m" if Styling.is_enabled() else ""
        
    @staticmethod
    def magenta():
        return "\033[45m" if Styling.is_enabled() else ""
        
    @staticmethod
    def cyan():
        return "\033[46m" if Styling.is_enabled() else ""
    
    @staticmethod
    def white():
        return "\033[47m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_black():
        return "\033[100m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_red():
        return "\033[101m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_green():
        return "\033[102m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_yellow():
        return "\033[103m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_blue():
        return "\033[104m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_magenta():
        return "\033[105m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_cyan():
        return "\033[106m" if Styling.is_enabled() else ""
    
    @staticmethod
    def br_white():
        return "\033[107m" if Styling.is_enabled() else ""


class Console:
    @staticmethod
    def clr_scr_to_end():
        '''Clear from cursor to end of screen'''
        return '\033[0J'
    
    @staticmethod
    def clr_scr_to_begin():
        '''Clear from cursor to beginning of screen'''
        return '\033[1J'
    
    @staticmethod
    def clr_entire_scr():
        '''Clear entire screen'''
        return '\033[2J'

    @staticmethod
    def clr_line_to_end():
        '''Clear from cursor to end of line'''
        return '\033[0K'

    @staticmethod
    def clr_line_to_begin():
        '''Clear from cursor to beginning of line'''
        return '\033[1K'

    @staticmethod
    def clr_entire_line():
        '''Clear entire line'''
        return '\033[2K'
    
    @staticmethod
    def scroll_up(n=1):
        '''
        Scroll the whole page up by n (default 1) lines.
        New lines are added at the bottom.
        '''
        if n > 0:
            return f'\033[{n}S'
        return ''

    @staticmethod
    def scroll_down(n=1):
        '''
        Scroll the whole page down by n (default 1) lines.
        New lines are added at the top.
        '''
        if n > 0:
            return f'\033[{n}T'
        return ''
    
    @staticmethod
    def bell():
        '''
        Sends a BEL (Bell) control character to make an audible noise.
        '''
        print('\x07', end="")
    
    @staticmethod
    def save_cursor():
        '''
        Save the current cursor position.
        Returns: ANSI escape code to save cursor position.
        '''
        return '\033[s'

    @staticmethod
    def restore_cursor():
        '''
        Restore the previously saved cursor position.
        Returns: ANSI escape code to restore cursor position.
        '''
        return '\033[u'
    
    @staticmethod
    def switch_alt_scr():
        '''
        Switch to the alternative screen buffer (CSI ? 1049 h).
        '''
        print('\033[?1049h', end='')

    @staticmethod
    def switch_orig_scr():
        '''
        Switch back to the original screen buffer (CSI ? 1049 l).
        '''
        print('\033[?1049l', end='')

    @staticmethod
    def show_cursor():
        '''
        Show the cursor (CSI ? 25 h).
        '''
        print('\033[?25h', end='')

    @staticmethod
    def hide_cursor():
        '''
        Hide the cursor (CSI ? 25 l).
        '''
        print('\033[?25l', end='')



class Styling:
    _enabled = True

    @classmethod
    def disable(cls):
        cls._enabled = False
        return ""

    @classmethod
    def enable(cls):
        cls._enabled = True
        return ""

    @classmethod
    def is_enabled(cls):
        return cls._enabled



from .styling import Styling

class Text:
    @staticmethod
    def black():
        return "\033[30m" if Styling.is_enabled() else ""
        
    @staticmethod
    def red():
        return "\033[31m" if Styling.is_enabled() else ""
        
    @staticmethod
    def green():
        return "\033[32m" if Styling.is_enabled() else ""
        
    @staticmethod
    def yellow():
        return "\033[33m" if Styling.is_enabled() else ""
        
    @staticmethod
    def blue():
        return "\033[34m" if Styling.is_enabled() else ""
        
    @staticmethod
    def magenta():
        return "\033[35m" if Styling.is_enabled() else ""
        
    @staticmethod
    def cyan():
        return "\033[36m" if Styling.is_enabled() else ""
    
    @staticmethod
    def white():
        return "\033[37m" if Styling.is_enabled() else ""
    
    @staticmethod
    def br_black():
        return "\033[90m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_red():
        return "\033[91m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_green():
        return "\033[92m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_yellow():
        return "\033[93m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_blue():
        return "\033[94m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_magenta():
        return "\033[95m" if Styling.is_enabled() else ""
        
    @staticmethod
    def br_cyan():
        return "\033[96m" if Styling.is_enabled() else ""
    
    @staticmethod
    def br_white():
        return "\033[97m" if Styling.is_enabled() else ""


from .Color import Color
from .styling import Styling

class Style:
    @staticmethod
    def reset(): #0m
        '''
        Reset or normal\n
        resets style and color of the text
        '''
        return "\033[0m" if Styling.is_enabled() else ""

    @staticmethod
    def bold(): #1m
        '''
        Bold or increased intensity\n
        As with faint, the color change is a PC (SCO / [CGA](https://en.wikipedia.org/wiki/Color_Graphics_Adapter))
        invention.[[38]](https://en.wikipedia.org/wiki/ANSI_escape_code#cite_note-SCO-39)\n
        '''
        return "\033[1m" if Styling.is_enabled() else ""
        
    @staticmethod
    def faint(): #2m
        '''
        Faint, decreased intensity, or dim\n
        May be implemented as a light [font weight](https://en.wikipedia.org/wiki/Font_weight) like bold.[[39]](https://en.wikipedia.org/wiki/ANSI_escape_code#cite_note-40)
        '''
        return "\033[2m" if Styling.is_enabled() else ""
        
    @staticmethod
    def italic(): #3m
        '''
        Italic\n
        Not widely supported. Sometimes treated as inverse or blink.[[38]](https://en.wikipedia.org/wiki/ANSI_escape_code#cite_note-SCO-39)\n
        '''
        return "\033[3m" if Styling.is_enabled() else ""
        
    @staticmethod
    def underlined(): #4m
        '''
        Underlined\n
        Style extensions exist for Kitty, VTE, mintty, iTerm2 and Konsole.[[40]](https://en.wikipedia.org/wiki/ANSI_escape_code#cite_note-color-u-41)
        [[41]](https://en.wikipedia.org/wiki/ANSI_escape_code#cite_note-color-u-kitty-spec-42)[[42]](https://en.wikipedia.org/wiki/ANSI_escape_code#cite_note-color-u-konsole-43)
        '''
        return "\033[4m" if Styling.is_enabled() else ""
        
    @staticmethod
    def slow_blink(): #5m
        '''
        Slow blink\n
        Sets blinking to less than 150 times per minute
        '''
        return "\033[5m" if Styling.is_enabled() else ""
        
    @staticmethod
    def rapid_blink(): #6m
        '''
        Rapid blink\n
        MS-DOS ANSI.SYS, 150+ per minute; not widely supported
        '''
        return "\033[6m" if Styling.is_enabled() else ""
        
    @staticmethod
    def reverse(): #7m
        '''
        [Reverse](https://en.wikipedia.org/wiki/Reverse_video) video or invert\n
        Swap foreground and background colors; inconsistent emulation[[43]](https://en.wikipedia.org/wiki/ANSI_escape_code#cite_note-console-termio-realize-44)
        [[dubious](https://en.wikipedia.org/wiki/Wikipedia:Accuracy_dispute#Disputed_statement) – [discuss](https://en.wikipedia.org/wiki/Talk:ANSI_escape_code#inconsistent_emulation)]
        '''
        return "\033[7m" if Styling.is_enabled() else ""
        
    @staticmethod
    def hidden(): #8m
        '''
        Conceal or hide\n
        Not widely supported.
        '''
        return "\033[8m" if Styling.is_enabled() else ""
        
    @staticmethod
    def strikethrough(): #9m
        '''
        [Crossed-out](https://en.wikipedia.org/wiki/Strikethrough), or strike\n
        Characters legible but marked as if for deletion. Not supported in Terminal.app.
        '''
        return "\033[9m" if Styling.is_enabled() else ""
    
    @staticmethod
    def doubly_underlined(): #21m
        '''
        Doubly underlined; or: not bold\n
        Double-underline per ECMA-48,[[5]](https://en.wikipedia.org/wiki/ANSI_escape_code#cite_note-ECMA-48-5): 8.3.117\n
        but instead disables bold intensity on several terminals,\n
        including in the [Linux kernel](https://en.wikipedia.org/wiki/Linux_kernel)'s
        [console](https://en.wikipedia.org/wiki/Linux_console) before version 4.17.[[44]](https://en.wikipedia.org/wiki/ANSI_escape_code#cite_note-45)
        '''
        return "\033[21m" if Styling.is_enabled() else ""
        
    @staticmethod
    def normal_intensity(): #22m
        '''
        Normal intensity\n
        Neither bold nor faint; color changes where intensity is implemented as such.
        '''
        return "\033[22m" if Styling.is_enabled() else ""

    @staticmethod
    def not_italic(): #23m
        '''
        Neither italic, nor blackletter
        '''
        return "\033[23m" if Styling.is_enabled() else ""

    @staticmethod
    def not_underlined(): #24m
        '''
        Not underlined\n
        Neither singly nor doubly underlined
        '''
        return "\033[24m" if Styling.is_enabled() else ""
        
    @staticmethod
    def not_blinking(): #25m
        '''
        Not blinking\n
        Turn blinking off
        '''
        return "\033[25m" if Styling.is_enabled() else ""
        
    @staticmethod
    def proportional_spacing(): #26m
        '''
        proportional_spacing\n
        [ITU T.61](https://en.wikipedia.org/wiki/ITU_T.61) and T.416, not known to be used on terminals
        '''
        return "\033[26m" if Styling.is_enabled() else ""
        
    @staticmethod
    def not_reversed(): #27m
        '''
        not reversed
        '''
        return "\033[27m" if Styling.is_enabled() else ""
        
    @staticmethod
    def reveal(): #28m
        '''
        Reveal\n
        Not concealed
        '''
        return "\033[28m" if Styling.is_enabled() else ""

    @staticmethod
    def not_strikethrough(): #29m
        '''
        Not crossed out
        '''
        return "\033[29m" if Styling.is_enabled() else ""
    
    @staticmethod
    def not_proportional_spacing(): #50m
        '''
        Disable proportional spacing\n
        T.61 and T.416
        '''
        return "\033[50m" if Styling.is_enabled() else ""
        
    @staticmethod
    def overlined(): #53m
        '''
        Overlined\n
        Not supported in Terminal.app
        '''
        return "\033[53m" if Styling.is_enabled() else ""
        
    @staticmethod
    def not_overlined(): #55m
        '''
        Not overlined\n
        '''
        return "\033[55m" if Styling.is_enabled() else ""
    
    @staticmethod
    def minecraft(*args):
        '''
        converts minecraft styled text to colored text\n
        Style.minecraft("&","&l&ahello &r&5world&e!&r")\n
        Style.minecraft("§l§ahello §r§5world§e!§r")
        '''
        if Styling.is_enabled():
            symbol = "§"

            codes = {
                f"0": Color.text("#000"),
                f"1": Color.text("#00a"),
                f"2": Color.text("#0a0"),
                f"3": Color.text("#0aa"),
                f"4": Color.text("#a00"),
                f"5": Color.text("#a0a"),
                f"6": Color.text("#fa0"),
                f"7": Color.text("#aaa"),
                f"8": Color.text("#555"),
                f"9": Color.text("#55f"),
                f"a": Color.text("#5f5"),
                f"b": Color.text("#5ff"),
                f"c": Color.text("#f55"),
                f"d": Color.text("#f5f"),
                f"e": Color.text("#ff5"),
                f"f": Color.text("#fff"),
                f"l": Style.bold(),
                f"m": Style.strikethrough(),
                f"n": Style.underlined(),
                f"o": Style.italic(),
                f"r": Style.reset()
                }

            if len(args) == 2:
                symbol, text = args
                for key, value in codes.items():
                    text = text.replace(symbol+key, value)

            if len(args) == 1:
                text = args
                for key, value in codes.items():
                    text = text.replace(symbol+key, value)
            return text
        else:
            return ""


#https://en.wikipedia.org/wiki/ANSI_escape_code

from .styling import Styling
from .Color import Color
from .Console import Console
from .Style import Style
from .ColorBackground import Background as background
from .ColorText import Text as text
import argparse

__all__ = ['Styling', 'Color', 'Style', 'Console']

def setup():
    try:
        import sys, os, platform
        from pkg_resources import get_distribution
        
        try:
            import requests

            current_version = get_distribution("chromaconsole").version
            latest_version = requests.get(f'https://pypi.org/pypi/chromaconsole/json').json()['info']['version']

            if tuple(map(int, current_version.split('.'))) < tuple(map(int, latest_version.split('.'))):
                os.system("pip install chromaconsole -U")
        except:
            pass

        def FixWinConsoleColors():
            if platform.system() == "Windows" and sys.stdout and hasattr(sys.stdout, "fileno"):
                try:
                    import ctypes
                    ctypes.windll.kernel32.SetConsoleMode(ctypes.windll.kernel32.GetStdHandle(-11), 7)
                except Exception:
                    pass

        FixWinConsoleColors()
    except:
        pass

setup()

def color_tuple(color_str):
    if not color_str:
        return ""

    if ',' in color_str:
        try:
            return tuple(map(int, color_str.replace(" ", "").lstrip('(').rstrip(')').split(',')))
        except ValueError:
            raise ValueError(f"Invalid RGB format: {color_str}")
    elif color_str.startswith('#'):
        color_str = color_str.lstrip("#")
        try:
            if len(color_str) == 3:
                r, g, b = (int(c * 2, 16) for c in color_str)
            elif len(color_str) == 6:
                r, g, b = (int(color_str[i:i+2], 16) for i in range(0, 6, 2))
            else:
                raise ValueError("Invalid color format")
        except ValueError:
            raise ValueError(f"Invalid hex color format: {color_str}")
        
        return (r, g, b)
    else:
        raise ValueError("Invalid color format")

def console_line(textcolor='', backgroundcolor=''):
    parser = argparse.ArgumentParser(description="Print colored text to the console.")
    parser.add_argument("-c", "--textcolor", default=textcolor, help="color code (e.g., #ff0000 or 255,0,0)")
    parser.add_argument("-b", "--backgroundcolor", default=backgroundcolor, help="color code (e.g., #ff0000 or 255,0,0)")
    parser.add_argument("-B", "--bold", action="store_true", help="Make the text bold")
    parser.add_argument("-I", "--italic", action="store_true", help="Make the text italic")
    parser.add_argument("-U", "--underline", action="store_true", help="Make the text underlined")
    parser.add_argument("-s", "--slowblink", action="store_true", help="Make the text slow blink")
    parser.add_argument("-r", "--rapidblink", action="store_true", help="Make the text rapid blink")
    parser.add_argument("-R", "--reverse", action="store_true", help="reverse bg and text color")
    parser.add_argument("-H", "--hidden", action="store_true", help="Make the text hidden")
    parser.add_argument("-S", "--strikethrough", action="store_true", help="Make the text strikethrough")
    parser.add_argument("-D", "--doublyunderlined", action="store_true", help="Make the text doubly underlined")
    parser.add_argument("-P", "--proportionalspacing", action="store_true", help="Make the text proportional spacing")
    parser.add_argument("-O", "--overlined", action="store_true", help="Make the text overlined")
    parser.add_argument("-X", "--reset", action="store_true", help="reset all to default")

    args = parser.parse_args()

    print(
        f"{Color.text(color_tuple(args.textcolor))}"
        f"{Color.background(color_tuple(args.backgroundcolor))}"
        f"{Style.bold() if args.bold else ''}"
        f"{Style.italic() if args.italic else ''}"
        f"{Style.underlined() if args.underline else ''}"
        f"{Style.slow_blink() if args.slowblink else ''}"
        f"{Style.rapid_blink() if args.rapidblink else ''}"
        f"{Style.reverse() if args.reverse else ''}"
        f"{Style.hidden() if args.hidden else ''}"
        f"{Style.strikethrough() if args.strikethrough else ''}"
        f"{Style.doubly_underlined() if args.doublyunderlined else ''}"
        f"{Style.proportional_spacing() if args.proportionalspacing else ''}"
        f"{Style.overlined() if args.overlined else ''}"
        f"{Style.reset() if args.reset else ''}")

def check_ver():
    try:
        from pkg_resources import get_distribution
        import requests
        print(f"current: {get_distribution('chromaconsole').version}\n latest: {requests.get(f'https://pypi.org/pypi/chromaconsole/json').json()['info']['version']}")
    except:
        print(f"Something went wrong")

if __name__ == "__main__":
    console_line()
    check_ver()



from .styling import Styling
from .ColorBackground import Background as background
from .ColorText import Text as text

def parse_color(*args):
    if len(args) == 1:
        if type(args[0]) == tuple:
            r, g, b = args[0]           #*  (rrr,ggg,bbb)
            return (r,g,b)
        else:
            color = args[0].lstrip("#")
            if "," in color:            #*  "rrr,ggg,bbb"
                color = color.replace(" ", "").replace("(", "").replace(")", "")
                color = color.split(",")
                r, g, b = map(int, color)
                return (r,g,b)
            if len(color) == 3:         #*  "#rgb"
                r, g, b = (int(c * 2, 16) for c in color)
                return (r,g,b)
            elif len(color) == 6:       #*  "#rrggbb"
                r, g, b = (int(color[i:i+2], 16) for i in range(0, 6, 2))
                return (r,g,b)
            else:
                raise ValueError(f"Invalid color format\n{args}")
    elif len(args) == 3:
        r, g, b = args
    else:
        raise ValueError(f"Invalid color format\n{args}")
    
def interpolate_color(start_color, end_color, factor):
    return tuple(int(start + (end - start) * factor) for start, end in zip(start_color, end_color))

class Color:
    class Text:
        '''
        Set foreground [color](https://en.wikipedia.org/wiki/ANSI_escape_code#3-bit_and_4-bit)
        '''
        black = staticmethod(text.black)
        red = staticmethod(text.red)
        green = staticmethod(text.green)
        yellow = staticmethod(text.yellow)
        blue = staticmethod(text.blue)
        magenta = staticmethod(text.magenta)
        cyan = staticmethod(text.cyan)
        white = staticmethod(text.white)
        br_black = staticmethod(text.br_black)
        br_red = staticmethod(text.br_red)
        br_green = staticmethod(text.br_green)
        br_yellow = staticmethod(text.br_yellow)
        br_blue = staticmethod(text.br_blue)
        br_magenta = staticmethod(text.br_magenta)
        br_cyan = staticmethod(text.br_cyan)
        br_white = staticmethod(text.br_white)
    class Background:
        '''
        Set background [color](https://en.wikipedia.org/wiki/ANSI_escape_code#3-bit_and_4-bit)
        '''
        black = staticmethod(background.black)
        red = staticmethod(background.red)
        green = staticmethod(background.green)
        yellow = staticmethod(background.yellow)
        blue = staticmethod(background.blue)
        magenta = staticmethod(background.magenta)
        cyan = staticmethod(background.cyan)
        white = staticmethod(background.white)
        br_black = staticmethod(background.br_black)
        br_red = staticmethod(background.br_red)
        br_green = staticmethod(background.br_green)
        br_yellow = staticmethod(background.br_yellow)
        br_blue = staticmethod(background.br_blue)
        br_magenta = staticmethod(background.br_magenta)
        br_cyan = staticmethod(background.br_cyan)
        br_white = staticmethod(background.br_white)

    @staticmethod
    def text(*args): #38;2;r;g;bm
        '''
        changes the color of the text\n
        Color.text(r, g, b)\n
        Color.text("#rrggbb")
        '''
        if args[0] == "":
            return ""
        if Styling.is_enabled():
            return f"\033[38;2;{parse_color(*args)[0]};{parse_color(*args)[1]};{parse_color(*args)[2]}m"
        else:
            return ""
        
    @staticmethod
    def text_gradient(text, start_color, end_color):
        '''
        this can be used to make text gradient\n
        text_gradient(str, (r,g,b) or "#rrggbb" , (r,g,b) or "#rrggbb")
        '''
        gradient_text = ""

        start_color = parse_color(start_color)
        end_color = parse_color(end_color)

        # Compute interpolation steps
        interpolation_step = 1 / (len(text) - 1)
        for i, char in enumerate(text):
            factor = i * interpolation_step
            interpolated_color = interpolate_color(start_color, end_color, factor)
            gradient_text += str(Color.text(interpolated_color)) + char

        return gradient_text + Color.default_text()
        
    @staticmethod
    def default_text(): #39m
        '''
        Default foreground color\n
        Implementation defined (according to standard)
        '''
        return "\033[39m" if Styling.is_enabled() else ""
    
    @staticmethod
    def background(*args): #48;2;r;g;bm
        '''
        changes the color of the text background\n
        Color.background(r, g, b)\n
        Color.background("#rrggbb")
        '''
        if args[0] == "":
            return ""
        if Styling.is_enabled():
            return f"\033[48;2;{parse_color(*args)[0]};{parse_color(*args)[1]};{parse_color(*args)[2]}m"
        else:
            return ""
        
    @staticmethod
    def background_gradient(text, start_color, end_color):
        '''
        this can be used to make background gradient\n
        background_gradient(str, (r,g,b) or "#rrggbb" , (r,g,b) or "#rrggbb")
        '''
        gradient_text = ""

        start_color = parse_color(start_color)
        end_color = parse_color(end_color)

        # Compute interpolation steps
        interpolation_step = 1 / (len(text) - 1)
        for i, char in enumerate(text):
            factor = i * interpolation_step
            interpolated_color = interpolate_color(start_color, end_color, factor)
            gradient_text += Color.background(interpolated_color) + char

        return gradient_text + Color.default_background()
    
    @staticmethod
    def default_background(): #49m
        '''
        Default background color\n
        Implementation defined (according to standard)
        '''
        return "\033[49m" if Styling.is_enabled() else ""